# 名词解释

`本地通知`:指点击 home 键，APP 进入后台，IM 通道还存在，消息通过 IM 通道接收

`推送`:指当接收方的主进程被杀死或者回收或者主动调用 disconnect 接口，导致 IM 通道与服务器断开，消息会通过推送下发

# 本地通知的实现

```
1.依赖 flutter_local_notifications
2.在 main.dart 实现并监听 WidgetsBindingObserver ，获取 APP 生命周期状态并保存
3.在 main.dart 中监听消息接收，如果在后台，发本地通知，否则通过 EventBus 通知会话列表和聊天页面收到了消息
```
详细可以参见该项目中 main.dart 的处理

# 远程推送的实现

IM Flutter Plugin 核心是 iOS/Android 的原生 SDK，所以远程推送与原生 SDK 的方式一致

[iOS 远程推送文档](https://www.rongcloud.cn/docs/ios_push.html)

[Android 远程推送文档](https://www.rongcloud.cn/docs/android_push.html)

# 常见问题

## iOS 如何获取 deviceToken

参考 Runner 中 `AppDelegate.m` 中的实现方法, 如果是 `Swift`, 请参考 [AppDelegate.swift](./AppDelegate.swift) 

## iOS 如何将远程推送的内容传递至 Flutter 层

参考 Runner 中 `AppDelegate.m` 中的 `sendDataToFlutter` 实现方法。如果是 `Swift`, 请参考 [AppDelegate.swift](./AppDelegate.swift) 


## Android 设备按返回键退出 APP，收不到消息，无法做本地通知

此时收到消息报如下错误 

```
D/FlutterNativeView(15798): FlutterView.send called on a detached view, channel=rongcloud_im_plugin
```

这个问题的原因可以看一下 [这个 Issue](https://github.com/flutter/flutter/issues/31734) ，上面提到的像是 Flutter 本身的问题

出现这个问题本质原因是 APP 被系统回收了，用来和 Android IMLib SDK 沟通的 MethodChannel 也被回收了，但是 Android IMLib 的 IM 通道还存在，此时有消息来， Android IMLib 通过 IM 通道能够收到，但是因为 MethodChannel 被销毁，无法通知给 Flutter APP，也就无法弹本地通知

这个问题目前的建议是当按返回键退出 APP 的时候，调用 disconnect 接口主动断开 IM 通道，这样就可以走推送通道